#include <Modules/modules.h>

char*
modules_names[] = {
	"dropshadow",	//0
	"clock",
	"temperature",
	"cpufreq",
	"battery",
	"pager",		//5
	"ibar",
	"ibox",
	"randr",
	"engage",
	"weather",		//10
	"snow",
	"start",
	"flame",
	"rain"			//14
};

char*
modules_description[] = {
	//0
	"Makes the windows drop a shadow on the desktop",
	"Displays a clock",
	"Displays the temperature",
	"Displays the CPU-Frequency",
	"Displays the battery-status",
	//5
	"Shows a panel which displays each desktop's windows",
	"Shows an iconbar to start Programs",
	"Shows minimized windows",
	"Resolution-changer",
	"Mac-OS inspired taskbar with additional functions",
	//10
	"Shows a panel which displays weather information",
	"Let it snow on your desktop",
	"A simple Start-Button, which opens the E-Menu",
	"Creates some flaming at the bottom of the screen",
	"Let it rain on your desktop"
};

void
modules_radio_clicked(Etk_Object *object, void *data){
	if (!modules_initialized)
		return;
	modules_switch			*module	= data;		
	Etk_Toggle_Button		*button	= ETK_TOGGLE_BUTTON(object);

	if (!(button->active)){
		char *command = get_raw_command();
		const char* label = etk_button_label_get(ETK_BUTTON(button));
		const char* module_name = etk_widget_name_get(module->name);
		if (!strcmp(label,"enabled")) {
			strcat(command,"module-load ");
			strcat(command,module_name);
			strcat(command," -module-enable ");
			strcat(command,module_name);
		} else if (!strcmp(label,"disabled")){
			strcat(command,"module-load ");
			strcat(command,module_name);
			strcat(command," -module-disable ");
			strcat(command,module_name);
		} else if (!strcmp(label,"unloaded")){
			strcat(command,"module-unload ");
			strcat(command,module_name);
		} else {
			fputs ("Weird, this shouldn't happen\n",stdout);
			return;
		}
		if (system(command))
			printf("Error on `%s`\n",command);
	}
}

void
modules_kill(){
	modules_initialized = FALSE;
	sub_window_kill();
}

modules_switch *
modules_switch_new(){
	modules_switch * new_switch = (modules_switch *)malloc(sizeof(modules_switch));
	new_switch->description = 
	new_switch->disabled =
	new_switch->enabled =
	new_switch->name = 
	new_switch->unloaded = NULL;
	return new_switch;
}

Etk_Widget*
init_modules(){
	modules_initialized = FALSE;
	Etk_Widget	*window = etk_window_new(),
				*table = etk_table_new(4,MODULEC*3-1,FALSE);
	int i;
	
	etk_window_title_set(ETK_WINDOW(window),"Modules-Settings");
	etk_signal_connect_after("destroy",ETK_OBJECT(window),ETK_CALLBACK(modules_kill),NULL);
	
	etk_container_add(ETK_CONTAINER(window),table);

	for (i=0;i<MODULEC;i++){
		char checkModule[512],temp[1024];
		char *listModule =  get_raw_command();
		int module_status = 0;
		modules_switch *switches = modules_switch_new();
		switches->description = NULL;
		
		sprintf(checkModule,"REPLY: \"%s\" ENABLED ",modules_names[i]);
		
		strcat(listModule,"module-list | grep ");
		strcat(listModule,modules_names[i]);
		
		FILE *op = popen(listModule,"r");
		
		temp[0] = 0;
		while (!feof(op)){
			fgets(temp,512,op);
			if (!strncmp(checkModule,temp,strlen(checkModule)))
				break;
			temp[0] = 0;
		}
		fclose(op);

		if (temp[0] == 0)
			module_status = -1;
		else if (temp[strlen(checkModule)] == '1')
			module_status = 1;
		else
			module_status = 0;
		
		sprintf(temp,"<+ font_size=15 align=right> %s ",modules_names[i]);
		//Name
		switches->name = etk_label_new(temp);
		etk_widget_name_set(switches->name,modules_names[i]);
		//Radios
		switches->enabled = etk_radio_button_new_with_label("enabled",NULL);
		switches->disabled = etk_radio_button_new_with_label_from_widget("disabled",ETK_RADIO_BUTTON(switches->enabled));
		switches->unloaded = etk_radio_button_new_with_label_from_widget("unloaded",ETK_RADIO_BUTTON(switches->enabled));
		sprintf(temp,"%s",modules_description[i]);
		switches->description = etk_label_new(temp);
		//Activate right switch
		switch (module_status) {
			case 1:
				etk_toggle_button_active_set(ETK_TOGGLE_BUTTON(switches->enabled),TRUE);
				break;
			case 0:
				etk_toggle_button_active_set(ETK_TOGGLE_BUTTON(switches->disabled),TRUE);
				break;
			default:
				etk_toggle_button_active_set(ETK_TOGGLE_BUTTON(switches->unloaded),TRUE);
		}
		//Connect Signals
		etk_signal_connect("clicked",ETK_OBJECT(switches->enabled),ETK_CALLBACK(modules_radio_clicked),switches);
		etk_signal_connect("clicked",ETK_OBJECT(switches->disabled),ETK_CALLBACK(modules_radio_clicked),switches);
		etk_signal_connect("clicked",ETK_OBJECT(switches->unloaded),ETK_CALLBACK(modules_radio_clicked),switches);
		//Add to table
		etk_table_attach(ETK_TABLE(table),switches->name		
			,0,0,i*3,i*3,0,0,ETK_FILL_POLICY_HEXPAND);
		etk_table_attach_defaults(ETK_TABLE(table),switches->enabled		,1,1,i*3,i*3);
		etk_table_attach_defaults(ETK_TABLE(table),switches->disabled	,2,2,i*3,i*3);
		etk_table_attach_defaults(ETK_TABLE(table),switches->unloaded	,3,3,i*3,i*3);
		//Add Description
		etk_table_attach_defaults(ETK_TABLE(table),switches->description, 0,3,i*3+1,i*3+1);
	}
	
	modules_initialized = TRUE;
	etk_widget_show_all(window);
	return window;
}
