#include <Modules/winlist.h>

char*
_winlist_options[] = {
	//setting
	"winlist-pos-align",	//-x/-y
	"winlist-pos-size",		//-w/-h
	"winlist-pos-min",		//-w/-h
	"winlist-pos-max",		//-w/-h
	//warp combo
	"winlist-warp-while-selecting",
	"winlist-warp-at-end",
	"winlist-warp-speed",
	//scroll combo
	"winlist-scroll-animate",
	"winlist-scroll-speed",
	//1|0
	"winlist-list-show-iconified",
	"winlist-list-show-other-desk-windows",
	"winlist-list-show-other-screen-windows",
	"winlist-list-uncover-while-selecting",
	"winlist-list-jump-desk-while-selecting",
	"winlist-list-focus-while-selecting",
	"winlist-list-raise-while-selecting"
},
*_winlist_options_labels[] = {
	"Position",
	"Size",
	"Minimum Size",
	"Maximum Size",
	"Warp while selecting",
	"Warp at end",
	"Warp speed",
	"Scroll animate",
	"Scroll speed",
	"Show iconified windows",
	"Show other desks' windows",
	"Show other screens' windows",
	"Uncover while selecting",
	"Jump desk while selecting",
	"Focus while selecting",
	"Raise while selecting"
},
*_winlist_options_help[] = {
	"",
	"",
	"",
	"",
	"Move mouse-pointer to window, currently selected <br> ",
	"Move mouse-pointer to window selected at the end <br> ",
	"How fast to move the mouse-pointer on warp (pix/sec) <br> ",
	"No idea <br> ",
	"",
	"Also show windows currently minimized, while selecting <br> ",
	"Also show windows from other desktops, while selecting <br> ",
	"Also show windows from other screens, while selecting <br> ",
	"Puts currently selected window on top of all other windows<br> ",
	"Jump to desktop, the currently selected window's at <br> ",
	"Give currently selected window focus <br> ",
	"Raise currently selected window"
};

char *
_winlist_get_setting(char *e_switch){
	char x,y,*setting1,*setting2,*option;
	if (!strcmp(e_switch,_winlist_options[0])){
		x = 'x';
		y = 'y';
	} else {
		x = 'w';
		y = 'h';
	}
	option = (char *) malloc(strlen(e_switch)+3);
	sprintf(option,"%s-%c",e_switch,x);
	setting1 = getSetting(option);
	sprintf(option,"%s-%c",e_switch,y);
	setting2 = getSetting(option);
	free(option);
	option = (char *) malloc(strlen(setting1) + strlen(setting2) + 4);
	sprintf(option,"%s X %s",setting1,setting2);
	return option;
}

void
_winlist_set(Etk_Object *object, void *data){
	Winlist_pos *wlp = data;
	char x,y,*option;
	const char *entry;
	option = (char *) malloc(strlen(wlp->e_switch)+3);
	if (!strcmp(wlp->e_switch,_winlist_options[0])){
		x = 'x'; y = 'y';
	} else {
		x = 'w'; y = 'h';
	}
	if ((entry = etk_entry_text_get(ETK_ENTRY(wlp->x_entry)))) {
		sprintf(option,"%s-%c",wlp->e_switch,x);
		setOption(option,entry);
	}
	if ((entry = etk_entry_text_get(ETK_ENTRY(wlp->y_entry)))) {
		sprintf(option,"%s-%c",wlp->e_switch,y);
		setOption(option,entry);
	}
	etk_label_set(ETK_LABEL(wlp->pos_label),_winlist_get_setting(wlp->e_switch));
}

Winlist_pos *
new_winlist_pos(char *label, char *e_switch, char *help_text){
	Winlist_pos * _new = (Winlist_pos *) malloc(sizeof(Winlist_pos));
	_new->label	= (char *) malloc(strlen(label)+1);
	strcpy(_new->label, label);
	_new->e_switch = (char *) malloc(strlen(e_switch)+1);
	strcpy(_new->e_switch, e_switch);
	_new->help_text = help_text;
	_new->pos_label = etk_label_new(_winlist_get_setting(e_switch));
	_new->x_entry = etk_entry_new();
	_new->y_entry = etk_entry_new();
	_new->button = etk_button_new_with_label("SET");
	etk_signal_connect("clicked",ETK_OBJECT(_new->button),ETK_CALLBACK(_winlist_set),_new);
	return _new;
}

int
_add_to_root_table_wlp(Etk_Widget *table, Winlist_pos *wlp, int row){
	Etk_Box *container = ETK_BOX(etk_hbox_new(FALSE,0));
	etk_box_pack_start(container,wlp->pos_label,0,0,0);
	etk_box_pack_end(container,wlp->button,0,0,0);
	etk_box_pack_end(container,wlp->y_entry,0,0,0);
	etk_box_pack_end(container,wlp->x_entry,0,0,0);
	return add_to_root_table(table,wlp->label,wlp->help_text,ETK_WIDGET(container),row);
}

char*
winlist_getSetting(char* option){
	char*	setting = malloc(64);
	char	option1[256],
			option2[256];
	if (option == _winlist_options[0]){
		sprintf(option1,"%s-x",option);
		sprintf(option2,"%s-y",option);
	}
	else{
		sprintf(option1,"%s-w",option);
		sprintf(option2,"%s-h",option);
	}
	sprintf(setting,"%s x %s",getSetting(option1),getSetting(option2));
	return setting;
}

Etk_Widget*
init_winlist(){
	Etk_Widget *window,*container,*table,*o_table;
	int i = 0;
	
	window = etk_window_new();
	etk_window_title_set(ETK_WINDOW(window),"Winlist Settings");
	etk_signal_connect_after("destroy",ETK_OBJECT(window),ETK_CALLBACK(sub_window_kill),NULL);
	
	container = etk_vbox_new(FALSE,3);
	etk_container_add(ETK_CONTAINER(window),container);
	
	table = etk_table_new(2,32,FALSE);
	etk_container_add(ETK_CONTAINER(container),table);

	//dimensions
	for ( i = 0; i < 4 ; i++)
		_add_to_root_table_wlp(	table,
								new_winlist_pos(	_winlist_options_labels[i],
													_winlist_options[i],
													_winlist_options_help[i]),
								i*2);
	//warp
	for ( i = 4; i < 6; i++){
		o_table = etk_table_new(1,1,FALSE);
		etk_table_attach(ETK_TABLE(o_table),get_prepared_toggle_button(_winlist_options[i])
			,0,0,0,0,0,0,ETK_FILL_POLICY_NONE);
		add_to_root_table(table,_winlist_options_labels[i],_winlist_options_help[i],o_table,i*2);
	}
	i=6;
	add_to_root_table_swb(	table,
							new_setting_with_buttons(	_winlist_options_labels[i],
														_winlist_options[i],
														_winlist_options_help[i],
														0.1,
														0),
							i*2);
	etk_table_attach_defaults(ETK_TABLE(table),etk_label_new(_winlist_options_help[i]),0,1,i*2+1,i*2+1);

	//scroll
	i++;
	o_table = etk_table_new(1,1,FALSE);
	etk_table_attach(ETK_TABLE(o_table),get_prepared_toggle_button(_winlist_options[i])
		,0,0,0,0,0,0,ETK_FILL_POLICY_NONE);
	add_to_root_table(table,_winlist_options_labels[i],_winlist_options_help[i],o_table,i*2);
	i++;
	add_to_root_table_swb(	table,
							new_setting_with_buttons(	_winlist_options_labels[i],
														_winlist_options[i],
														_winlist_options_help[i],
														0.1,
														0),
							i*2);
	//rest
	for ( i++; i < 16; i++){
		o_table = etk_hbox_new(FALSE,1);
		etk_box_pack_start(ETK_BOX(o_table),get_prepared_toggle_button(_winlist_options[i]),FALSE,FALSE,1);
		add_to_root_table(table,_winlist_options_labels[i],_winlist_options_help[i],o_table,i*2);
		etk_table_attach_defaults(ETK_TABLE(table),etk_label_new(_winlist_options_help[i]),0,1,i*2+1,i*2+1);
	}

	etk_widget_show_all(window);
	return window;
}
